import os
import pytest
import nbformat
from nbclient import NotebookClient
from nbclient.exceptions import CellExecutionError
import requests

import json
import time
from urllib.parse import urlparse

import websocket


from urllib.parse import urljoin

# setup data
@pytest.fixture(scope="session", autouse=True)
def prepare_data():
    # Load basketballplayer Dataset
    os.system(
        "docker run --rm --network setup_ngai-net vesoft/nebula-console:v3"
        " -addr graphd -port 9669 -u root -p nebula -e \":play basketballplayer\"")

    # Create label_propagation Tag
    os.system(
        "docker run --rm --network setup_ngai-net vesoft/nebula-console:v3"
        " -addr graphd -port 9669 -u root -p nebula -e "
        "\" USE basketballplayer; "
        "CREATE TAG IF NOT EXISTS label_propagation "
        "(cluster_id string NOT NULL);\"")

    # create and cleanup run directory
    os.system("mkdir -p tests/integration/setup/run")
    os.system("rm -rf tests/integration/setup/run/*")
    os.system("pwd")

    # copy notebooks to run directory
    os.system("cp -r tests/integration/notebooks/* tests/integration/setup/run/")

    print("Setup data done...")

def run_cell_and_get_result(jupyter_base_url, kernel_id, code, token):
    ws_url = jupyter_base_url.replace("http", "ws") + f"api/kernels/{kernel_id}/channels?token={token}"
    ws = websocket.create_connection(ws_url)

    # 发送执行请求
    execute_request = {
        "header": {
            "msg_id": "execute_1",
            "msg_type": "execute_request",
            "username": "",
            "session": "",
            "version": "5.2",
        },
        "content": {
            "code": code,
            "silent": False,
            "store_history": True,
            "user_expressions": {},
            "allow_stdin": False,
            "stop_on_error": True,
        },
        "metadata": {},
        "buffers": [],
    }
    ws.send(json.dumps(execute_request))

    # 等待执行结果
    result = None
    while True:
        msg_str = ws.recv()
        msg = json.loads(msg_str)

        msg_type = msg["msg_type"]
        if msg_type == "execute_result" or msg_type == "error":
            result = msg["content"]
            break
        elif msg_type == "status" and msg["content"]["execution_state"] == "idle":
            break

    ws.close()

    return result


def test_notebook_execution():

jupyter_base_url = "http://127.0.0.1:28888/"
api_base_url = urljoin(jupyter_base_url, "api/")
notebook_path = "run/spark_engine_e2e.ipynb"
token = "nebula"
session = requests.Session()
session.headers.update({"Authorization": f"token {token}"})

notebook_url = urljoin(api_base_url, f"contents/{notebook_path}")
response = session.get(notebook_url)
response.raise_for_status()
notebook_data = response.json()

kernels_url = urljoin(api_base_url, "kernels")
response = session.post(kernels_url, json={"name": "python3"})
response.raise_for_status()
kernel_data = response.json()
kernel_id = kernel_data["id"]

cells = nbformat.from_dict(notebook_data["content"])["cells"]

for cell in cells:
    if cell["cell_type"] == "code":
        code = cell["source"]
        result = run_cell_and_get_result(jupyter_base_url, kernel_id, code, token)
        print(result)
